@@ -1,6 +1,6 @@
 /**************************************************************************
  * 
- * Copyright 2007 Tungsten Graphics, Inc., Cedar Park, Texas.
+ * Copyright 2007 VMware, Inc.
  * Copyright (c) 2008-2010 VMware, Inc.
  * All Rights Reserved.
  * 
@@ -19,7 +19,7 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
- * IN NO EVENT SHALL TUNGSTEN GRAPHICS AND/OR ITS SUPPLIERS BE LIABLE FOR
+ * IN NO EVENT SHALL VMWARE AND/OR ITS SUPPLIERS BE LIABLE FOR
  * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
  * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
@@ -45,6 +45,7 @@
 #include "pipe/p_defines.h"
 #include "pipe/p_screen.h"
 #include "util/u_format.h"
+#include "st_cb_texture.h"
 #include "st_context.h"
 #include "st_format.h"
 
@@ -53,73 +54,75 @@
  * Translate Mesa format to Gallium format.
  */
 enum pipe_format
-st_mesa_format_to_pipe_format(gl_format mesaFormat)
+st_mesa_format_to_pipe_format(mesa_format mesaFormat)
 {
    switch (mesaFormat) {
-   case MESA_FORMAT_RGBA8888:
+   case MESA_FORMAT_A8B8G8R8_UNORM:
       return PIPE_FORMAT_ABGR8888_UNORM;
-   case MESA_FORMAT_RGBA8888_REV:
+   case MESA_FORMAT_R8G8B8A8_UNORM:
       return PIPE_FORMAT_RGBA8888_UNORM;
-   case MESA_FORMAT_ARGB8888:
+   case MESA_FORMAT_B8G8R8A8_UNORM:
       return PIPE_FORMAT_BGRA8888_UNORM;
-   case MESA_FORMAT_ARGB8888_REV:
+   case MESA_FORMAT_A8R8G8B8_UNORM:
       return PIPE_FORMAT_ARGB8888_UNORM;
-   case MESA_FORMAT_RGBX8888:
+   case MESA_FORMAT_X8B8G8R8_UNORM:
       return PIPE_FORMAT_XBGR8888_UNORM;
-   case MESA_FORMAT_RGBX8888_REV:
+   case MESA_FORMAT_R8G8B8X8_UNORM:
       return PIPE_FORMAT_RGBX8888_UNORM;
-   case MESA_FORMAT_XRGB8888:
+   case MESA_FORMAT_B8G8R8X8_UNORM:
       return PIPE_FORMAT_BGRX8888_UNORM;
-   case MESA_FORMAT_XRGB8888_REV:
+   case MESA_FORMAT_X8R8G8B8_UNORM:
       return PIPE_FORMAT_XRGB8888_UNORM;
-   case MESA_FORMAT_ARGB1555:
+   case MESA_FORMAT_B5G5R5A1_UNORM:
       return PIPE_FORMAT_B5G5R5A1_UNORM;
-   case MESA_FORMAT_ARGB4444:
+   case MESA_FORMAT_B4G4R4A4_UNORM:
       return PIPE_FORMAT_B4G4R4A4_UNORM;
-   case MESA_FORMAT_RGB565:
+   case MESA_FORMAT_B5G6R5_UNORM:
       return PIPE_FORMAT_B5G6R5_UNORM;
-   case MESA_FORMAT_RGB332:
+   case MESA_FORMAT_B2G3R3_UNORM:
       return PIPE_FORMAT_B2G3R3_UNORM;
-   case MESA_FORMAT_ARGB2101010:
+   case MESA_FORMAT_B10G10R10A2_UNORM:
       return PIPE_FORMAT_B10G10R10A2_UNORM;
-   case MESA_FORMAT_AL44:
+   case MESA_FORMAT_L4A4_UNORM:
       return PIPE_FORMAT_L4A4_UNORM;
-   case MESA_FORMAT_AL88:
+   case MESA_FORMAT_L8A8_UNORM:
       return PIPE_FORMAT_L8A8_UNORM;
-   case MESA_FORMAT_AL1616:
+   case MESA_FORMAT_L16A16_UNORM:
       return PIPE_FORMAT_L16A16_UNORM;
-   case MESA_FORMAT_A8:
+   case MESA_FORMAT_A_UNORM8:
       return PIPE_FORMAT_A8_UNORM;
-   case MESA_FORMAT_A16:
+   case MESA_FORMAT_A_UNORM16:
       return PIPE_FORMAT_A16_UNORM;
-   case MESA_FORMAT_L8:
+   case MESA_FORMAT_L_UNORM8:
       return PIPE_FORMAT_L8_UNORM;
-   case MESA_FORMAT_L16:
+   case MESA_FORMAT_L_UNORM16:
       return PIPE_FORMAT_L16_UNORM;
-   case MESA_FORMAT_I8:
+   case MESA_FORMAT_I_UNORM8:
       return PIPE_FORMAT_I8_UNORM;
-   case MESA_FORMAT_I16:
+   case MESA_FORMAT_I_UNORM16:
       return PIPE_FORMAT_I16_UNORM;
-   case MESA_FORMAT_Z16:
+   case MESA_FORMAT_Z_UNORM16:
       return PIPE_FORMAT_Z16_UNORM;
-   case MESA_FORMAT_Z32:
+   case MESA_FORMAT_Z_UNORM32:
       return PIPE_FORMAT_Z32_UNORM;
-   case MESA_FORMAT_Z24_S8:
+   case MESA_FORMAT_S8_UINT_Z24_UNORM:
       return PIPE_FORMAT_S8_UINT_Z24_UNORM;
-   case MESA_FORMAT_S8_Z24:
+   case MESA_FORMAT_Z24_UNORM_S8_UINT:
       return PIPE_FORMAT_Z24_UNORM_S8_UINT;
-   case MESA_FORMAT_Z24_X8:
+   case MESA_FORMAT_X8_UINT_Z24_UNORM:
       return PIPE_FORMAT_X8Z24_UNORM;
-   case MESA_FORMAT_X8_Z24:
+   case MESA_FORMAT_Z24_UNORM_X8_UINT:
       return PIPE_FORMAT_Z24X8_UNORM;
-   case MESA_FORMAT_S8:
+   case MESA_FORMAT_S_UINT8:
       return PIPE_FORMAT_S8_UINT;
-   case MESA_FORMAT_Z32_FLOAT:
+   case MESA_FORMAT_Z_FLOAT32:
       return PIPE_FORMAT_Z32_FLOAT;
-   case MESA_FORMAT_Z32_FLOAT_X24S8:
+   case MESA_FORMAT_Z32_FLOAT_S8X24_UINT:
       return PIPE_FORMAT_Z32_FLOAT_S8X24_UINT;
    case MESA_FORMAT_YCBCR:
       return PIPE_FORMAT_UYVY;
+   case MESA_FORMAT_YCBCR_REV:
+      return PIPE_FORMAT_YUYV;
    case MESA_FORMAT_RGB_DXT1:
       return PIPE_FORMAT_DXT1_RGB;
    case MESA_FORMAT_RGBA_DXT1:
@@ -136,16 +139,18 @@ st_mesa_format_to_pipe_format(gl_format mesaFormat)
       return PIPE_FORMAT_DXT3_SRGBA;
    case MESA_FORMAT_SRGBA_DXT5:
       return PIPE_FORMAT_DXT5_SRGBA;
-   case MESA_FORMAT_SLA8:
+   case MESA_FORMAT_L8A8_SRGB:
       return PIPE_FORMAT_L8A8_SRGB;
-   case MESA_FORMAT_SL8:
+   case MESA_FORMAT_L_SRGB8:
       return PIPE_FORMAT_L8_SRGB;
-   case MESA_FORMAT_SRGB8:
+   case MESA_FORMAT_BGR_SRGB8:
       return PIPE_FORMAT_R8G8B8_SRGB;
-   case MESA_FORMAT_SRGBA8:
+   case MESA_FORMAT_A8B8G8R8_SRGB:
       return PIPE_FORMAT_A8B8G8R8_SRGB;
-   case MESA_FORMAT_SARGB8:
+   case MESA_FORMAT_B8G8R8A8_SRGB:
       return PIPE_FORMAT_B8G8R8A8_SRGB;
+   case MESA_FORMAT_R8G8B8A8_SRGB:
+      return PIPE_FORMAT_R8G8B8A8_SRGB;
    case MESA_FORMAT_RGBA_FLOAT32:
       return PIPE_FORMAT_R32G32B32A32_FLOAT;
    case MESA_FORMAT_RGBA_FLOAT16:
@@ -154,21 +159,21 @@ st_mesa_format_to_pipe_format(gl_format mesaFormat)
       return PIPE_FORMAT_R32G32B32_FLOAT;
    case MESA_FORMAT_RGB_FLOAT16:
       return PIPE_FORMAT_R16G16B16_FLOAT;
-   case MESA_FORMAT_LUMINANCE_ALPHA_FLOAT32:
+   case MESA_FORMAT_LA_FLOAT32:
       return PIPE_FORMAT_L32A32_FLOAT;
-   case MESA_FORMAT_LUMINANCE_ALPHA_FLOAT16:
+   case MESA_FORMAT_LA_FLOAT16:
       return PIPE_FORMAT_L16A16_FLOAT;
-   case MESA_FORMAT_LUMINANCE_FLOAT32:
+   case MESA_FORMAT_L_FLOAT32:
       return PIPE_FORMAT_L32_FLOAT;
-   case MESA_FORMAT_LUMINANCE_FLOAT16:
+   case MESA_FORMAT_L_FLOAT16:
       return PIPE_FORMAT_L16_FLOAT;
-   case MESA_FORMAT_ALPHA_FLOAT32:
+   case MESA_FORMAT_A_FLOAT32:
       return PIPE_FORMAT_A32_FLOAT;
-   case MESA_FORMAT_ALPHA_FLOAT16:
+   case MESA_FORMAT_A_FLOAT16:
       return PIPE_FORMAT_A16_FLOAT;
-   case MESA_FORMAT_INTENSITY_FLOAT32:
+   case MESA_FORMAT_I_FLOAT32:
       return PIPE_FORMAT_I32_FLOAT;
-   case MESA_FORMAT_INTENSITY_FLOAT16:
+   case MESA_FORMAT_I_FLOAT16:
       return PIPE_FORMAT_I16_FLOAT;
    case MESA_FORMAT_R_FLOAT32:
       return PIPE_FORMAT_R32_FLOAT;
@@ -179,97 +184,97 @@ st_mesa_format_to_pipe_format(gl_format mesaFormat)
    case MESA_FORMAT_RG_FLOAT16:
       return PIPE_FORMAT_R16G16_FLOAT;
 
-   case MESA_FORMAT_R8:
+   case MESA_FORMAT_R_UNORM8:
       return PIPE_FORMAT_R8_UNORM;
-   case MESA_FORMAT_R16:
+   case MESA_FORMAT_R_UNORM16:
       return PIPE_FORMAT_R16_UNORM;
-   case MESA_FORMAT_GR88:
+   case MESA_FORMAT_R8G8_UNORM:
       return PIPE_FORMAT_R8G8_UNORM;
-   case MESA_FORMAT_GR1616:
+   case MESA_FORMAT_R16G16_UNORM:
       return PIPE_FORMAT_R16G16_UNORM;
-   case MESA_FORMAT_RGBA_16:
+   case MESA_FORMAT_RGBA_UNORM16:
       return PIPE_FORMAT_R16G16B16A16_UNORM;
 
    /* signed int formats */
-   case MESA_FORMAT_ALPHA_UINT8:
+   case MESA_FORMAT_A_UINT8:
       return PIPE_FORMAT_A8_UINT;
-   case MESA_FORMAT_ALPHA_UINT16:
+   case MESA_FORMAT_A_UINT16:
       return PIPE_FORMAT_A16_UINT;
-   case MESA_FORMAT_ALPHA_UINT32:
+   case MESA_FORMAT_A_UINT32:
       return PIPE_FORMAT_A32_UINT;
 
-   case MESA_FORMAT_ALPHA_INT8:
+   case MESA_FORMAT_A_SINT8:
       return PIPE_FORMAT_A8_SINT;
-   case MESA_FORMAT_ALPHA_INT16:
+   case MESA_FORMAT_A_SINT16:
       return PIPE_FORMAT_A16_SINT;
-   case MESA_FORMAT_ALPHA_INT32:
+   case MESA_FORMAT_A_SINT32:
       return PIPE_FORMAT_A32_SINT;
 
-   case MESA_FORMAT_INTENSITY_UINT8:
+   case MESA_FORMAT_I_UINT8:
       return PIPE_FORMAT_I8_UINT;
-   case MESA_FORMAT_INTENSITY_UINT16:
+   case MESA_FORMAT_I_UINT16:
       return PIPE_FORMAT_I16_UINT;
-   case MESA_FORMAT_INTENSITY_UINT32:
+   case MESA_FORMAT_I_UINT32:
       return PIPE_FORMAT_I32_UINT;
 
-   case MESA_FORMAT_INTENSITY_INT8:
+   case MESA_FORMAT_I_SINT8:
       return PIPE_FORMAT_I8_SINT;
-   case MESA_FORMAT_INTENSITY_INT16:
+   case MESA_FORMAT_I_SINT16:
       return PIPE_FORMAT_I16_SINT;
-   case MESA_FORMAT_INTENSITY_INT32:
+   case MESA_FORMAT_I_SINT32:
       return PIPE_FORMAT_I32_SINT;
 
-   case MESA_FORMAT_LUMINANCE_UINT8:
+   case MESA_FORMAT_L_UINT8:
       return PIPE_FORMAT_L8_UINT;
-   case MESA_FORMAT_LUMINANCE_UINT16:
+   case MESA_FORMAT_L_UINT16:
       return PIPE_FORMAT_L16_UINT;
-   case MESA_FORMAT_LUMINANCE_UINT32:
+   case MESA_FORMAT_L_UINT32:
       return PIPE_FORMAT_L32_UINT;
 
-   case MESA_FORMAT_LUMINANCE_INT8:
+   case MESA_FORMAT_L_SINT8:
       return PIPE_FORMAT_L8_SINT;
-   case MESA_FORMAT_LUMINANCE_INT16:
+   case MESA_FORMAT_L_SINT16:
       return PIPE_FORMAT_L16_SINT;
-   case MESA_FORMAT_LUMINANCE_INT32:
+   case MESA_FORMAT_L_SINT32:
       return PIPE_FORMAT_L32_SINT;
 
-   case MESA_FORMAT_LUMINANCE_ALPHA_UINT8:
+   case MESA_FORMAT_LA_UINT8:
       return PIPE_FORMAT_L8A8_UINT;
-   case MESA_FORMAT_LUMINANCE_ALPHA_UINT16:
+   case MESA_FORMAT_LA_UINT16:
       return PIPE_FORMAT_L16A16_UINT;
-   case MESA_FORMAT_LUMINANCE_ALPHA_UINT32:
+   case MESA_FORMAT_LA_UINT32:
       return PIPE_FORMAT_L32A32_UINT;
 
-   case MESA_FORMAT_LUMINANCE_ALPHA_INT8:
+   case MESA_FORMAT_LA_SINT8:
       return PIPE_FORMAT_L8A8_SINT;
-   case MESA_FORMAT_LUMINANCE_ALPHA_INT16:
+   case MESA_FORMAT_LA_SINT16:
       return PIPE_FORMAT_L16A16_SINT;
-   case MESA_FORMAT_LUMINANCE_ALPHA_INT32:
+   case MESA_FORMAT_LA_SINT32:
       return PIPE_FORMAT_L32A32_SINT;
 
-   case MESA_FORMAT_R_INT8:
+   case MESA_FORMAT_R_SINT8:
       return PIPE_FORMAT_R8_SINT;
-   case MESA_FORMAT_RG_INT8:
+   case MESA_FORMAT_RG_SINT8:
       return PIPE_FORMAT_R8G8_SINT;
-   case MESA_FORMAT_RGB_INT8:
+   case MESA_FORMAT_RGB_SINT8:
       return PIPE_FORMAT_R8G8B8_SINT;
-   case MESA_FORMAT_RGBA_INT8:
+   case MESA_FORMAT_RGBA_SINT8:
       return PIPE_FORMAT_R8G8B8A8_SINT;
-   case MESA_FORMAT_R_INT16:
+   case MESA_FORMAT_R_SINT16:
       return PIPE_FORMAT_R16_SINT;
-   case MESA_FORMAT_RG_INT16:
+   case MESA_FORMAT_RG_SINT16:
       return PIPE_FORMAT_R16G16_SINT;
-   case MESA_FORMAT_RGB_INT16:
+   case MESA_FORMAT_RGB_SINT16:
       return PIPE_FORMAT_R16G16B16_SINT;
-   case MESA_FORMAT_RGBA_INT16:
+   case MESA_FORMAT_RGBA_SINT16:
       return PIPE_FORMAT_R16G16B16A16_SINT;
-   case MESA_FORMAT_R_INT32:
+   case MESA_FORMAT_R_SINT32:
       return PIPE_FORMAT_R32_SINT;
-   case MESA_FORMAT_RG_INT32:
+   case MESA_FORMAT_RG_SINT32:
       return PIPE_FORMAT_R32G32_SINT;
-   case MESA_FORMAT_RGB_INT32:
+   case MESA_FORMAT_RGB_SINT32:
       return PIPE_FORMAT_R32G32B32_SINT;
-   case MESA_FORMAT_RGBA_INT32:
+   case MESA_FORMAT_RGBA_SINT32:
       return PIPE_FORMAT_R32G32B32A32_SINT;
 
    /* unsigned int formats */
@@ -298,98 +303,103 @@ st_mesa_format_to_pipe_format(gl_format mesaFormat)
    case MESA_FORMAT_RGBA_UINT32:
       return PIPE_FORMAT_R32G32B32A32_UINT;
 
-   case MESA_FORMAT_RED_RGTC1:
+   case MESA_FORMAT_R_RGTC1_UNORM:
       return PIPE_FORMAT_RGTC1_UNORM;
-   case MESA_FORMAT_SIGNED_RED_RGTC1:
+   case MESA_FORMAT_R_RGTC1_SNORM:
       return PIPE_FORMAT_RGTC1_SNORM;
-   case MESA_FORMAT_RG_RGTC2:
+   case MESA_FORMAT_RG_RGTC2_UNORM:
       return PIPE_FORMAT_RGTC2_UNORM;
-   case MESA_FORMAT_SIGNED_RG_RGTC2:
+   case MESA_FORMAT_RG_RGTC2_SNORM:
       return PIPE_FORMAT_RGTC2_SNORM;
 
-   case MESA_FORMAT_L_LATC1:
+   case MESA_FORMAT_L_LATC1_UNORM:
       return PIPE_FORMAT_LATC1_UNORM;
-   case MESA_FORMAT_SIGNED_L_LATC1:
+   case MESA_FORMAT_L_LATC1_SNORM:
       return PIPE_FORMAT_LATC1_SNORM;
-   case MESA_FORMAT_LA_LATC2:
+   case MESA_FORMAT_LA_LATC2_UNORM:
       return PIPE_FORMAT_LATC2_UNORM;
-   case MESA_FORMAT_SIGNED_LA_LATC2:
+   case MESA_FORMAT_LA_LATC2_SNORM:
       return PIPE_FORMAT_LATC2_SNORM;
 
    case MESA_FORMAT_ETC1_RGB8:
       return PIPE_FORMAT_ETC1_RGB8;
 
    /* signed normalized formats */
-   case MESA_FORMAT_SIGNED_R8:
+   case MESA_FORMAT_R_SNORM8:
       return PIPE_FORMAT_R8_SNORM;
-   case MESA_FORMAT_SIGNED_RG88_REV:
+   case MESA_FORMAT_R8G8_SNORM:
       return PIPE_FORMAT_R8G8_SNORM;
-   case MESA_FORMAT_SIGNED_RGBA8888_REV:
+   case MESA_FORMAT_R8G8B8A8_SNORM:
       return PIPE_FORMAT_R8G8B8A8_SNORM;
 
-   case MESA_FORMAT_SIGNED_A8:
+   case MESA_FORMAT_A_SNORM8:
       return PIPE_FORMAT_A8_SNORM;
-   case MESA_FORMAT_SIGNED_L8:
+   case MESA_FORMAT_L_SNORM8:
       return PIPE_FORMAT_L8_SNORM;
-   case MESA_FORMAT_SIGNED_AL88:
+   case MESA_FORMAT_L8A8_SNORM:
       return PIPE_FORMAT_L8A8_SNORM;
-   case MESA_FORMAT_SIGNED_I8:
+   case MESA_FORMAT_I_SNORM8:
       return PIPE_FORMAT_I8_SNORM;
 
-   case MESA_FORMAT_SIGNED_R16:
+   case MESA_FORMAT_R_SNORM16:
       return PIPE_FORMAT_R16_SNORM;
-   case MESA_FORMAT_SIGNED_GR1616:
+   case MESA_FORMAT_R16G16_SNORM:
       return PIPE_FORMAT_R16G16_SNORM;
-   case MESA_FORMAT_SIGNED_RGBA_16:
+   case MESA_FORMAT_RGBA_SNORM16:
       return PIPE_FORMAT_R16G16B16A16_SNORM;
 
-   case MESA_FORMAT_SIGNED_A16:
+   case MESA_FORMAT_A_SNORM16:
       return PIPE_FORMAT_A16_SNORM;
-   case MESA_FORMAT_SIGNED_L16:
+   case MESA_FORMAT_L_SNORM16:
       return PIPE_FORMAT_L16_SNORM;
-   case MESA_FORMAT_SIGNED_AL1616:
+   case MESA_FORMAT_LA_SNORM16:
       return PIPE_FORMAT_L16A16_SNORM;
-   case MESA_FORMAT_SIGNED_I16:
+   case MESA_FORMAT_I_SNORM16:
       return PIPE_FORMAT_I16_SNORM;
 
-   case MESA_FORMAT_RGB9_E5_FLOAT:
+   case MESA_FORMAT_R9G9B9E5_FLOAT:
       return PIPE_FORMAT_R9G9B9E5_FLOAT;
-   case MESA_FORMAT_R11_G11_B10_FLOAT:
+   case MESA_FORMAT_R11G11B10_FLOAT:
       return PIPE_FORMAT_R11G11B10_FLOAT;
-   case MESA_FORMAT_ARGB2101010_UINT:
+   case MESA_FORMAT_B10G10R10A2_UINT:
       return PIPE_FORMAT_B10G10R10A2_UINT;
+   case MESA_FORMAT_R10G10B10A2_UINT:
+      return PIPE_FORMAT_R10G10B10A2_UINT;
 
-   case MESA_FORMAT_XRGB4444_UNORM:
+   case MESA_FORMAT_B4G4R4X4_UNORM:
       return PIPE_FORMAT_B4G4R4X4_UNORM;
-   case MESA_FORMAT_XRGB1555_UNORM:
+   case MESA_FORMAT_B5G5R5X1_UNORM:
       return PIPE_FORMAT_B5G5R5X1_UNORM;
-   case MESA_FORMAT_XBGR8888_SNORM:
+   case MESA_FORMAT_R8G8B8X8_SNORM:
       return PIPE_FORMAT_R8G8B8X8_SNORM;
-   case MESA_FORMAT_XBGR8888_SRGB:
+   case MESA_FORMAT_R8G8B8X8_SRGB:
       return PIPE_FORMAT_R8G8B8X8_SRGB;
-   case MESA_FORMAT_XBGR8888_UINT:
+   case MESA_FORMAT_RGBX_UINT8:
       return PIPE_FORMAT_R8G8B8X8_UINT;
-   case MESA_FORMAT_XBGR8888_SINT:
+   case MESA_FORMAT_RGBX_SINT8:
       return PIPE_FORMAT_R8G8B8X8_SINT;
-   case MESA_FORMAT_XRGB2101010_UNORM:
+   case MESA_FORMAT_B10G10R10X2_UNORM:
       return PIPE_FORMAT_B10G10R10X2_UNORM;
-   case MESA_FORMAT_XBGR16161616_UNORM:
+   case MESA_FORMAT_RGBX_UNORM16:
       return PIPE_FORMAT_R16G16B16X16_UNORM;
-   case MESA_FORMAT_XBGR16161616_SNORM:
+   case MESA_FORMAT_RGBX_SNORM16:
       return PIPE_FORMAT_R16G16B16X16_SNORM;
-   case MESA_FORMAT_XBGR16161616_FLOAT:
+   case MESA_FORMAT_RGBX_FLOAT16:
       return PIPE_FORMAT_R16G16B16X16_FLOAT;
-   case MESA_FORMAT_XBGR16161616_UINT:
+   case MESA_FORMAT_RGBX_UINT16:
       return PIPE_FORMAT_R16G16B16X16_UINT;
-   case MESA_FORMAT_XBGR16161616_SINT:
+   case MESA_FORMAT_RGBX_SINT16:
       return PIPE_FORMAT_R16G16B16X16_SINT;
-   case MESA_FORMAT_XBGR32323232_FLOAT:
+   case MESA_FORMAT_RGBX_FLOAT32:
       return PIPE_FORMAT_R32G32B32X32_FLOAT;
-   case MESA_FORMAT_XBGR32323232_UINT:
+   case MESA_FORMAT_RGBX_UINT32:
       return PIPE_FORMAT_R32G32B32X32_UINT;
-   case MESA_FORMAT_XBGR32323232_SINT:
+   case MESA_FORMAT_RGBX_SINT32:
       return PIPE_FORMAT_R32G32B32X32_SINT;
 
+   case MESA_FORMAT_B8G8R8X8_SRGB:
+      return PIPE_FORMAT_B8G8R8X8_SRGB;
+
    default:
       return PIPE_FORMAT_NONE;
    }
@@ -399,76 +409,76 @@ st_mesa_format_to_pipe_format(gl_format mesaFormat)
 /**
  * Translate Gallium format to Mesa format.
  */
-gl_format
+mesa_format
 st_pipe_format_to_mesa_format(enum pipe_format format)
 {
    switch (format) {
    case PIPE_FORMAT_ABGR8888_UNORM:
-      return MESA_FORMAT_RGBA8888;
+      return MESA_FORMAT_A8B8G8R8_UNORM;
    case PIPE_FORMAT_RGBA8888_UNORM:
-      return MESA_FORMAT_RGBA8888_REV;
+      return MESA_FORMAT_R8G8B8A8_UNORM;
    case PIPE_FORMAT_BGRA8888_UNORM:
-      return MESA_FORMAT_ARGB8888;
+      return MESA_FORMAT_B8G8R8A8_UNORM;
    case PIPE_FORMAT_ARGB8888_UNORM:
-      return MESA_FORMAT_ARGB8888_REV;
+      return MESA_FORMAT_A8R8G8B8_UNORM;
    case PIPE_FORMAT_XBGR8888_UNORM:
-      return MESA_FORMAT_RGBX8888;
+      return MESA_FORMAT_X8B8G8R8_UNORM;
    case PIPE_FORMAT_RGBX8888_UNORM:
-      return MESA_FORMAT_RGBX8888_REV;
+      return MESA_FORMAT_R8G8B8X8_UNORM;
    case PIPE_FORMAT_BGRX8888_UNORM:
-      return MESA_FORMAT_XRGB8888;
+      return MESA_FORMAT_B8G8R8X8_UNORM;
    case PIPE_FORMAT_XRGB8888_UNORM:
-      return MESA_FORMAT_XRGB8888_REV;
+      return MESA_FORMAT_X8R8G8B8_UNORM;
    case PIPE_FORMAT_B5G5R5A1_UNORM:
-      return MESA_FORMAT_ARGB1555;
+      return MESA_FORMAT_B5G5R5A1_UNORM;
    case PIPE_FORMAT_B4G4R4A4_UNORM:
-      return MESA_FORMAT_ARGB4444;
+      return MESA_FORMAT_B4G4R4A4_UNORM;
    case PIPE_FORMAT_B5G6R5_UNORM:
-      return MESA_FORMAT_RGB565;
+      return MESA_FORMAT_B5G6R5_UNORM;
    case PIPE_FORMAT_B2G3R3_UNORM:
-      return MESA_FORMAT_RGB332;
+      return MESA_FORMAT_B2G3R3_UNORM;
    case PIPE_FORMAT_B10G10R10A2_UNORM:
-      return MESA_FORMAT_ARGB2101010;
+      return MESA_FORMAT_B10G10R10A2_UNORM;
    case PIPE_FORMAT_L4A4_UNORM:
-      return MESA_FORMAT_AL44;
+      return MESA_FORMAT_L4A4_UNORM;
    case PIPE_FORMAT_L8A8_UNORM:
-      return MESA_FORMAT_AL88;
+      return MESA_FORMAT_L8A8_UNORM;
    case PIPE_FORMAT_L16A16_UNORM:
-      return MESA_FORMAT_AL1616;
+      return MESA_FORMAT_L16A16_UNORM;
    case PIPE_FORMAT_A8_UNORM:
-      return MESA_FORMAT_A8;
+      return MESA_FORMAT_A_UNORM8;
    case PIPE_FORMAT_A16_UNORM:
-      return MESA_FORMAT_A16;
+      return MESA_FORMAT_A_UNORM16;
    case PIPE_FORMAT_L8_UNORM:
-      return MESA_FORMAT_L8;
+      return MESA_FORMAT_L_UNORM8;
    case PIPE_FORMAT_L16_UNORM:
-      return MESA_FORMAT_L16;
+      return MESA_FORMAT_L_UNORM16;
    case PIPE_FORMAT_I8_UNORM:
-      return MESA_FORMAT_I8;
+      return MESA_FORMAT_I_UNORM8;
    case PIPE_FORMAT_I16_UNORM:
-      return MESA_FORMAT_I16;
+      return MESA_FORMAT_I_UNORM16;
    case PIPE_FORMAT_S8_UINT:
-      return MESA_FORMAT_S8;
+      return MESA_FORMAT_S_UINT8;
 
    case PIPE_FORMAT_R16G16B16A16_UNORM:
-      return MESA_FORMAT_RGBA_16;
+      return MESA_FORMAT_RGBA_UNORM16;
 
    case PIPE_FORMAT_Z16_UNORM:
-      return MESA_FORMAT_Z16;
+      return MESA_FORMAT_Z_UNORM16;
    case PIPE_FORMAT_Z32_UNORM:
-      return MESA_FORMAT_Z32;
+      return MESA_FORMAT_Z_UNORM32;
    case PIPE_FORMAT_S8_UINT_Z24_UNORM:
-      return MESA_FORMAT_Z24_S8;
+      return MESA_FORMAT_S8_UINT_Z24_UNORM;
    case PIPE_FORMAT_X8Z24_UNORM:
-      return MESA_FORMAT_Z24_X8;
+      return MESA_FORMAT_X8_UINT_Z24_UNORM;
    case PIPE_FORMAT_Z24X8_UNORM:
-      return MESA_FORMAT_X8_Z24;
+      return MESA_FORMAT_Z24_UNORM_X8_UINT;
    case PIPE_FORMAT_Z24_UNORM_S8_UINT:
-      return MESA_FORMAT_S8_Z24;
+      return MESA_FORMAT_Z24_UNORM_S8_UINT;
    case PIPE_FORMAT_Z32_FLOAT:
-      return MESA_FORMAT_Z32_FLOAT;
+      return MESA_FORMAT_Z_FLOAT32;
    case PIPE_FORMAT_Z32_FLOAT_S8X24_UINT:
-      return MESA_FORMAT_Z32_FLOAT_X24S8;
+      return MESA_FORMAT_Z32_FLOAT_S8X24_UINT;
 
    case PIPE_FORMAT_UYVY:
       return MESA_FORMAT_YCBCR;
@@ -492,15 +502,15 @@ st_pipe_format_to_mesa_format(enum pipe_format format)
    case PIPE_FORMAT_DXT5_SRGBA:
       return MESA_FORMAT_SRGBA_DXT5;
    case PIPE_FORMAT_L8A8_SRGB:
-      return MESA_FORMAT_SLA8;
+      return MESA_FORMAT_L8A8_SRGB;
    case PIPE_FORMAT_L8_SRGB:
-      return MESA_FORMAT_SL8;
+      return MESA_FORMAT_L_SRGB8;
    case PIPE_FORMAT_R8G8B8_SRGB:
-      return MESA_FORMAT_SRGB8;
+      return MESA_FORMAT_BGR_SRGB8;
    case PIPE_FORMAT_A8B8G8R8_SRGB:
-      return MESA_FORMAT_SRGBA8;
+      return MESA_FORMAT_A8B8G8R8_SRGB;
    case PIPE_FORMAT_B8G8R8A8_SRGB:
-      return MESA_FORMAT_SARGB8;
+      return MESA_FORMAT_B8G8R8A8_SRGB;
    case PIPE_FORMAT_R32G32B32A32_FLOAT:
       return MESA_FORMAT_RGBA_FLOAT32;
    case PIPE_FORMAT_R16G16B16A16_FLOAT:
@@ -510,21 +520,21 @@ st_pipe_format_to_mesa_format(enum pipe_format format)
    case PIPE_FORMAT_R16G16B16_FLOAT:
       return MESA_FORMAT_RGB_FLOAT16;
    case PIPE_FORMAT_L32A32_FLOAT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_FLOAT32;
+      return MESA_FORMAT_LA_FLOAT32;
    case PIPE_FORMAT_L16A16_FLOAT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_FLOAT16;
+      return MESA_FORMAT_LA_FLOAT16;
    case PIPE_FORMAT_L32_FLOAT:
-      return MESA_FORMAT_LUMINANCE_FLOAT32;
+      return MESA_FORMAT_L_FLOAT32;
    case PIPE_FORMAT_L16_FLOAT:
-      return MESA_FORMAT_LUMINANCE_FLOAT16;
+      return MESA_FORMAT_L_FLOAT16;
    case PIPE_FORMAT_A32_FLOAT:
-      return MESA_FORMAT_ALPHA_FLOAT32;
+      return MESA_FORMAT_A_FLOAT32;
    case PIPE_FORMAT_A16_FLOAT:
-      return MESA_FORMAT_ALPHA_FLOAT16;
+      return MESA_FORMAT_A_FLOAT16;
    case PIPE_FORMAT_I32_FLOAT:
-      return MESA_FORMAT_INTENSITY_FLOAT32;
+      return MESA_FORMAT_I_FLOAT32;
    case PIPE_FORMAT_I16_FLOAT:
-      return MESA_FORMAT_INTENSITY_FLOAT16;
+      return MESA_FORMAT_I_FLOAT16;
    case PIPE_FORMAT_R32_FLOAT:
       return MESA_FORMAT_R_FLOAT32;
    case PIPE_FORMAT_R16_FLOAT:
@@ -535,92 +545,92 @@ st_pipe_format_to_mesa_format(enum pipe_format format)
       return MESA_FORMAT_RG_FLOAT16;
 
    case PIPE_FORMAT_R8_UNORM:
-      return MESA_FORMAT_R8;
+      return MESA_FORMAT_R_UNORM8;
    case PIPE_FORMAT_R16_UNORM:
-      return MESA_FORMAT_R16;
+      return MESA_FORMAT_R_UNORM16;
    case PIPE_FORMAT_R8G8_UNORM:
-      return MESA_FORMAT_GR88;
+      return MESA_FORMAT_R8G8_UNORM;
    case PIPE_FORMAT_R16G16_UNORM:
-      return MESA_FORMAT_GR1616;
+      return MESA_FORMAT_R16G16_UNORM;
 
    case PIPE_FORMAT_A8_UINT:
-      return MESA_FORMAT_ALPHA_UINT8;
+      return MESA_FORMAT_A_UINT8;
    case PIPE_FORMAT_A16_UINT:
-      return MESA_FORMAT_ALPHA_UINT16;
+      return MESA_FORMAT_A_UINT16;
    case PIPE_FORMAT_A32_UINT:
-      return MESA_FORMAT_ALPHA_UINT32;
+      return MESA_FORMAT_A_UINT32;
    case PIPE_FORMAT_A8_SINT:
-      return MESA_FORMAT_ALPHA_INT8;
+      return MESA_FORMAT_A_SINT8;
    case PIPE_FORMAT_A16_SINT:
-      return MESA_FORMAT_ALPHA_INT16;
+      return MESA_FORMAT_A_SINT16;
    case PIPE_FORMAT_A32_SINT:
-      return MESA_FORMAT_ALPHA_INT32;
+      return MESA_FORMAT_A_SINT32;
 
    case PIPE_FORMAT_I8_UINT:
-      return MESA_FORMAT_INTENSITY_UINT8;
+      return MESA_FORMAT_I_UINT8;
    case PIPE_FORMAT_I16_UINT:
-      return MESA_FORMAT_INTENSITY_UINT16;
+      return MESA_FORMAT_I_UINT16;
    case PIPE_FORMAT_I32_UINT:
-      return MESA_FORMAT_INTENSITY_UINT32;
+      return MESA_FORMAT_I_UINT32;
    case PIPE_FORMAT_I8_SINT:
-      return MESA_FORMAT_INTENSITY_INT8;
+      return MESA_FORMAT_I_SINT8;
    case PIPE_FORMAT_I16_SINT:
-      return MESA_FORMAT_INTENSITY_INT16;
+      return MESA_FORMAT_I_SINT16;
    case PIPE_FORMAT_I32_SINT:
-      return MESA_FORMAT_INTENSITY_INT32;
+      return MESA_FORMAT_I_SINT32;
 
   case PIPE_FORMAT_L8_UINT:
-      return MESA_FORMAT_LUMINANCE_UINT8;
+      return MESA_FORMAT_L_UINT8;
    case PIPE_FORMAT_L16_UINT:
-      return MESA_FORMAT_LUMINANCE_UINT16;
+      return MESA_FORMAT_L_UINT16;
    case PIPE_FORMAT_L32_UINT:
-      return MESA_FORMAT_LUMINANCE_UINT32;
+      return MESA_FORMAT_L_UINT32;
    case PIPE_FORMAT_L8_SINT:
-      return MESA_FORMAT_LUMINANCE_INT8;
+      return MESA_FORMAT_L_SINT8;
    case PIPE_FORMAT_L16_SINT:
-      return MESA_FORMAT_LUMINANCE_INT16;
+      return MESA_FORMAT_L_SINT16;
    case PIPE_FORMAT_L32_SINT:
-      return MESA_FORMAT_LUMINANCE_INT32;
+      return MESA_FORMAT_L_SINT32;
 
    case PIPE_FORMAT_L8A8_UINT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_UINT8;
+      return MESA_FORMAT_LA_UINT8;
    case PIPE_FORMAT_L16A16_UINT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_UINT16;
+      return MESA_FORMAT_LA_UINT16;
    case PIPE_FORMAT_L32A32_UINT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_UINT32;
+      return MESA_FORMAT_LA_UINT32;
    case PIPE_FORMAT_L8A8_SINT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_INT8;
+      return MESA_FORMAT_LA_SINT8;
    case PIPE_FORMAT_L16A16_SINT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_INT16;
+      return MESA_FORMAT_LA_SINT16;
    case PIPE_FORMAT_L32A32_SINT:
-      return MESA_FORMAT_LUMINANCE_ALPHA_INT32;
+      return MESA_FORMAT_LA_SINT32;
 
    case PIPE_FORMAT_R8_SINT:
-      return MESA_FORMAT_R_INT8;
+      return MESA_FORMAT_R_SINT8;
    case PIPE_FORMAT_R8G8_SINT:
-      return MESA_FORMAT_RG_INT8;
+      return MESA_FORMAT_RG_SINT8;
    case PIPE_FORMAT_R8G8B8_SINT:
-      return MESA_FORMAT_RGB_INT8;
+      return MESA_FORMAT_RGB_SINT8;
    case PIPE_FORMAT_R8G8B8A8_SINT:
-      return MESA_FORMAT_RGBA_INT8;
+      return MESA_FORMAT_RGBA_SINT8;
 
    case PIPE_FORMAT_R16_SINT:
-      return MESA_FORMAT_R_INT16;
+      return MESA_FORMAT_R_SINT16;
    case PIPE_FORMAT_R16G16_SINT:
-      return MESA_FORMAT_RG_INT16;
+      return MESA_FORMAT_RG_SINT16;
    case PIPE_FORMAT_R16G16B16_SINT:
-      return MESA_FORMAT_RGB_INT16;
+      return MESA_FORMAT_RGB_SINT16;
    case PIPE_FORMAT_R16G16B16A16_SINT:
-      return MESA_FORMAT_RGBA_INT16;
+      return MESA_FORMAT_RGBA_SINT16;
 
    case PIPE_FORMAT_R32_SINT:
-      return MESA_FORMAT_R_INT32;
+      return MESA_FORMAT_R_SINT32;
    case PIPE_FORMAT_R32G32_SINT:
-      return MESA_FORMAT_RG_INT32;
+      return MESA_FORMAT_RG_SINT32;
    case PIPE_FORMAT_R32G32B32_SINT:
-      return MESA_FORMAT_RGB_INT32;
+      return MESA_FORMAT_RGB_SINT32;
    case PIPE_FORMAT_R32G32B32A32_SINT:
-      return MESA_FORMAT_RGBA_INT32;
+      return MESA_FORMAT_RGBA_SINT32;
 
    /* unsigned int formats */
    case PIPE_FORMAT_R8_UINT:
@@ -651,105 +661,140 @@ st_pipe_format_to_mesa_format(enum pipe_format format)
       return MESA_FORMAT_RGBA_UINT32;
 
    case PIPE_FORMAT_RGTC1_UNORM:
-      return MESA_FORMAT_RED_RGTC1;
+      return MESA_FORMAT_R_RGTC1_UNORM;
    case PIPE_FORMAT_RGTC1_SNORM:
-      return MESA_FORMAT_SIGNED_RED_RGTC1;
+      return MESA_FORMAT_R_RGTC1_SNORM;
    case PIPE_FORMAT_RGTC2_UNORM:
-      return MESA_FORMAT_RG_RGTC2;
+      return MESA_FORMAT_RG_RGTC2_UNORM;
    case PIPE_FORMAT_RGTC2_SNORM:
-      return MESA_FORMAT_SIGNED_RG_RGTC2;
+      return MESA_FORMAT_RG_RGTC2_SNORM;
 
    case PIPE_FORMAT_LATC1_UNORM:
-      return MESA_FORMAT_L_LATC1;
+      return MESA_FORMAT_L_LATC1_UNORM;
    case PIPE_FORMAT_LATC1_SNORM:
-      return MESA_FORMAT_SIGNED_L_LATC1;
+      return MESA_FORMAT_L_LATC1_SNORM;
    case PIPE_FORMAT_LATC2_UNORM:
-      return MESA_FORMAT_LA_LATC2;
+      return MESA_FORMAT_LA_LATC2_UNORM;
    case PIPE_FORMAT_LATC2_SNORM:
-      return MESA_FORMAT_SIGNED_LA_LATC2;
+      return MESA_FORMAT_LA_LATC2_SNORM;
 
    case PIPE_FORMAT_ETC1_RGB8:
       return MESA_FORMAT_ETC1_RGB8;
 
    /* signed normalized formats */
    case PIPE_FORMAT_R8_SNORM:
-      return MESA_FORMAT_SIGNED_R8;
+      return MESA_FORMAT_R_SNORM8;
    case PIPE_FORMAT_R8G8_SNORM:
-      return MESA_FORMAT_SIGNED_RG88_REV;
+      return MESA_FORMAT_R8G8_SNORM;
    case PIPE_FORMAT_R8G8B8A8_SNORM:
-      return MESA_FORMAT_SIGNED_RGBA8888_REV;
+      return MESA_FORMAT_R8G8B8A8_SNORM;
 
    case PIPE_FORMAT_A8_SNORM:
-      return MESA_FORMAT_SIGNED_A8;
+      return MESA_FORMAT_A_SNORM8;
    case PIPE_FORMAT_L8_SNORM:
-      return MESA_FORMAT_SIGNED_L8;
+      return MESA_FORMAT_L_SNORM8;
    case PIPE_FORMAT_L8A8_SNORM:
-      return MESA_FORMAT_SIGNED_AL88;
+      return MESA_FORMAT_L8A8_SNORM;
    case PIPE_FORMAT_I8_SNORM:
-      return MESA_FORMAT_SIGNED_I8;
+      return MESA_FORMAT_I_SNORM8;
 
    case PIPE_FORMAT_R16_SNORM:
-      return MESA_FORMAT_SIGNED_R16;
+      return MESA_FORMAT_R_SNORM16;
    case PIPE_FORMAT_R16G16_SNORM:
-      return MESA_FORMAT_SIGNED_GR1616;
+      return MESA_FORMAT_R16G16_SNORM;
    case PIPE_FORMAT_R16G16B16A16_SNORM:
-      return MESA_FORMAT_SIGNED_RGBA_16;
+      return MESA_FORMAT_RGBA_SNORM16;
 
    case PIPE_FORMAT_A16_SNORM:
-      return MESA_FORMAT_SIGNED_A16;
+      return MESA_FORMAT_A_SNORM16;
    case PIPE_FORMAT_L16_SNORM:
-      return MESA_FORMAT_SIGNED_L16;
+      return MESA_FORMAT_L_SNORM16;
    case PIPE_FORMAT_L16A16_SNORM:
-      return MESA_FORMAT_SIGNED_AL1616;
+      return MESA_FORMAT_LA_SNORM16;
    case PIPE_FORMAT_I16_SNORM:
-      return MESA_FORMAT_SIGNED_I16;
+      return MESA_FORMAT_I_SNORM16;
 
    case PIPE_FORMAT_R9G9B9E5_FLOAT:
-      return MESA_FORMAT_RGB9_E5_FLOAT;
+      return MESA_FORMAT_R9G9B9E5_FLOAT;
    case PIPE_FORMAT_R11G11B10_FLOAT:
-      return MESA_FORMAT_R11_G11_B10_FLOAT;
+      return MESA_FORMAT_R11G11B10_FLOAT;
 
    case PIPE_FORMAT_B10G10R10A2_UINT:
-      return MESA_FORMAT_ARGB2101010_UINT;
+      return MESA_FORMAT_B10G10R10A2_UINT;
+   case PIPE_FORMAT_R10G10B10A2_UINT:
+      return MESA_FORMAT_R10G10B10A2_UINT;
 
    case PIPE_FORMAT_B4G4R4X4_UNORM:
-      return MESA_FORMAT_XRGB4444_UNORM;
+      return MESA_FORMAT_B4G4R4X4_UNORM;
    case PIPE_FORMAT_B5G5R5X1_UNORM:
-      return MESA_FORMAT_XRGB1555_UNORM;
+      return MESA_FORMAT_B5G5R5X1_UNORM;
    case PIPE_FORMAT_R8G8B8X8_SNORM:
-      return MESA_FORMAT_XBGR8888_SNORM;
+      return MESA_FORMAT_R8G8B8X8_SNORM;
    case PIPE_FORMAT_R8G8B8X8_SRGB:
-      return MESA_FORMAT_XBGR8888_SRGB;
+      return MESA_FORMAT_R8G8B8X8_SRGB;
    case PIPE_FORMAT_R8G8B8X8_UINT:
-      return MESA_FORMAT_XBGR8888_UINT;
+      return MESA_FORMAT_RGBX_UINT8;
    case PIPE_FORMAT_R8G8B8X8_SINT:
-      return MESA_FORMAT_XBGR8888_SINT;
+      return MESA_FORMAT_RGBX_SINT8;
    case PIPE_FORMAT_B10G10R10X2_UNORM:
-      return MESA_FORMAT_XRGB2101010_UNORM;
+      return MESA_FORMAT_B10G10R10X2_UNORM;
    case PIPE_FORMAT_R16G16B16X16_UNORM:
-      return MESA_FORMAT_XBGR16161616_UNORM;
+      return MESA_FORMAT_RGBX_UNORM16;
    case PIPE_FORMAT_R16G16B16X16_SNORM:
-      return MESA_FORMAT_XBGR16161616_SNORM;
+      return MESA_FORMAT_RGBX_SNORM16;
    case PIPE_FORMAT_R16G16B16X16_FLOAT:
-      return MESA_FORMAT_XBGR16161616_FLOAT;
+      return MESA_FORMAT_RGBX_FLOAT16;
    case PIPE_FORMAT_R16G16B16X16_UINT:
-      return MESA_FORMAT_XBGR16161616_UINT;
+      return MESA_FORMAT_RGBX_UINT16;
    case PIPE_FORMAT_R16G16B16X16_SINT:
-      return MESA_FORMAT_XBGR16161616_SINT;
+      return MESA_FORMAT_RGBX_SINT16;
    case PIPE_FORMAT_R32G32B32X32_FLOAT:
-      return MESA_FORMAT_XBGR32323232_FLOAT;
+      return MESA_FORMAT_RGBX_FLOAT32;
    case PIPE_FORMAT_R32G32B32X32_UINT:
-      return MESA_FORMAT_XBGR32323232_UINT;
+      return MESA_FORMAT_RGBX_UINT32;
    case PIPE_FORMAT_R32G32B32X32_SINT:
-      return MESA_FORMAT_XBGR32323232_SINT;
+      return MESA_FORMAT_RGBX_SINT32;
+
+   case PIPE_FORMAT_B8G8R8X8_SRGB:
+      return MESA_FORMAT_B8G8R8X8_SRGB;
+   case PIPE_FORMAT_R8G8B8A8_SRGB:
+      return MESA_FORMAT_R8G8B8A8_SRGB;
 
    default:
-      assert(0);
       return MESA_FORMAT_NONE;
    }
 }
 
 
+/**
+ * Debug only: check that the two functions above correctly map
+ * Mesa formats to Gallium formats and back again.
+ */
+static void
+test_format_conversion(void)
+{
+   GLuint i;
+
+   /* test all Mesa formats */
+   for (i = 1; i < MESA_FORMAT_COUNT; i++) {
+      enum pipe_format pf = st_mesa_format_to_pipe_format(i);
+      if (pf != PIPE_FORMAT_NONE) {
+         mesa_format mf = st_pipe_format_to_mesa_format(pf);
+         assert(mf == i);
+      }
+   }
+
+   /* Test all Gallium formats */
+   for (i = 1; i < PIPE_FORMAT_COUNT; i++) {
+      mesa_format mf = st_pipe_format_to_mesa_format(i);
+      if (mf != MESA_FORMAT_NONE) {
+         enum pipe_format pf = st_mesa_format_to_pipe_format(mf);
+         assert(pf == i);
+      }
+   }
+}
+
+
 /**
  * Map GL texture formats to Gallium pipe formats.
  */
@@ -1001,7 +1046,8 @@ static const struct format_mapping format_map[] = {
    /* sRGB formats */
    {
       { GL_SRGB_EXT, GL_SRGB8_EXT, 0 },
-      { PIPE_FORMAT_R8G8B8X8_SRGB, DEFAULT_SRGBA_FORMATS }
+      { PIPE_FORMAT_R8G8B8X8_SRGB, PIPE_FORMAT_B8G8R8X8_SRGB,
+        DEFAULT_SRGBA_FORMATS }
    },
    {
       { GL_SRGB_ALPHA_EXT, GL_SRGB8_ALPHA8_EXT, 0 },
@@ -1010,7 +1056,7 @@ static const struct format_mapping format_map[] = {
    {
       { GL_COMPRESSED_SRGB_EXT, GL_COMPRESSED_SRGB_S3TC_DXT1_EXT, 0 },
       { PIPE_FORMAT_DXT1_SRGB, PIPE_FORMAT_R8G8B8X8_SRGB,
-        DEFAULT_SRGBA_FORMATS }
+        PIPE_FORMAT_B8G8R8X8_SRGB, DEFAULT_SRGBA_FORMATS }
    },
    {
       { GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT, 0 },
@@ -1482,7 +1528,7 @@ static const struct format_mapping format_map[] = {
    },
    {
       { GL_RGB10_A2UI, 0 },
-      { PIPE_FORMAT_B10G10R10A2_UINT, 0 }
+      { PIPE_FORMAT_R10G10B10A2_UINT, PIPE_FORMAT_B10G10R10A2_UINT, 0 }
    },
 };
 
@@ -1625,6 +1671,18 @@ st_choose_format(struct st_context *st, GLenum internalFormat,
    int i, j;
    enum pipe_format pf;
 
+#ifdef DEBUG
+   {
+      static boolean firstCall = TRUE;
+      if (firstCall) {
+         test_format_conversion();
+         firstCall = FALSE;
+      }
+   }
+#else
+   (void) test_format_conversion;
+#endif
+
    /* can't render to compressed formats at this time */
    if (_mesa_is_compressed_format(st->ctx, internalFormat)
        && (bindings & ~PIPE_BIND_SAMPLER_VIEW)) {
@@ -1686,12 +1744,18 @@ enum pipe_format
 st_choose_matching_format(struct pipe_screen *screen, unsigned bind,
 			  GLenum format, GLenum type, GLboolean swapBytes)
 {
-   gl_format mesa_format;
+   mesa_format mesa_format;
 
    for (mesa_format = 1; mesa_format < MESA_FORMAT_COUNT; mesa_format++) {
       if (_mesa_get_format_color_encoding(mesa_format) == GL_SRGB) {
          continue;
       }
+      if (_mesa_get_format_bits(mesa_format, GL_TEXTURE_INTENSITY_SIZE) > 0) {
+         /* If `format` is GL_RED/GL_RED_INTEGER, then we might match some
+          * intensity formats, which we don't want.
+          */
+         continue;
+      }
 
       if (_mesa_format_matches_format_and_type(mesa_format, format, type,
                                                swapBytes)) {
@@ -1713,7 +1777,7 @@ st_choose_matching_format(struct pipe_screen *screen, unsigned bind,
 /**
  * Called via ctx->Driver.ChooseTextureFormat().
  */
-gl_format
+mesa_format
 st_ChooseTextureFormat(struct gl_context *ctx, GLenum target,
                        GLint internalFormat,
                        GLenum format, GLenum type)
@@ -1726,6 +1790,7 @@ st_ChooseTextureFormat(struct gl_context *ctx, GLenum target,
    struct st_context *st = st_context(ctx);
    enum pipe_format pFormat;
    unsigned bindings;
+   enum pipe_texture_target pTarget = gl_target_to_pipe(target);
 
    if (target == GL_TEXTURE_1D || target == GL_TEXTURE_1D_ARRAY) {
       /* We don't do compression for these texture targets because of
@@ -1782,12 +1847,12 @@ st_ChooseTextureFormat(struct gl_context *ctx, GLenum target,
    }
 
    pFormat = st_choose_format(st, internalFormat, format, type,
-                              PIPE_TEXTURE_2D, 0, bindings, ctx->Mesa_DXTn);
+                              pTarget, 0, bindings, ctx->Mesa_DXTn);
 
    if (pFormat == PIPE_FORMAT_NONE) {
       /* try choosing format again, this time without render target bindings */
       pFormat = st_choose_format(st, internalFormat, format, type,
-                                 PIPE_TEXTURE_2D, 0, PIPE_BIND_SAMPLER_VIEW,
+                                 pTarget, 0, PIPE_BIND_SAMPLER_VIEW,
                                  ctx->Mesa_DXTn);
    }
 
@@ -1852,12 +1917,12 @@ st_QuerySamplesForFormat(struct gl_context *ctx, GLenum target,
  * Similarly for texture border colors.
  */
 void
-st_translate_color(union gl_color_union *colorIn,
+st_translate_color(const union gl_color_union *colorIn,
                    union pipe_color_union *colorOut,
                    GLenum baseFormat, GLboolean is_integer)
 {
    if (is_integer) {
-      int *in = colorIn->i;
+      const int *in = colorIn->i;
       int *out = colorOut->i;
 
       switch (baseFormat) {
@@ -1899,7 +1964,7 @@ st_translate_color(union gl_color_union *colorIn,
       }
    }
    else {
-      float *in = colorIn->f;
+      const float *in = colorIn->f;
       float *out = colorOut->f;
 
       switch (baseFormat) {
